﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;
using System.Web;
using System.Web.UI;

namespace MySoapLib {
    public class HttpAsynchOperation : IAsyncResult {
        #region private fields
        private bool _completed;
        private readonly Object _state;
        private readonly AsyncCallback _callback;
        private readonly HttpContext _ht = null;
        SoapServerProtocol protocol = new SoapServerProtocol();
        #endregion

        #region IAsyncResult member
        bool IAsyncResult.IsCompleted { get { return _completed; } }
        WaitHandle IAsyncResult.AsyncWaitHandle { get { return null; } }
        Object IAsyncResult.AsyncState { get { return _state; } }
        bool IAsyncResult.CompletedSynchronously { get { return false; } }
        #endregion

        public HttpAsynchOperation(AsyncCallback callback, HttpContext context, Object state) {
            _callback = callback;
            _ht = context;
            _state = state;
            _completed = false;
        }
        public void StartAsyncWork(object obj) {
            ThreadPool.QueueUserWorkItem(StartAsyncTask, obj);
        }
        public void WriteResult(object result) {
            MemoryStream stream = new MemoryStream();
            protocol.WriteReturns(new object[] { result }, stream);
            string output = Encoding.UTF8.GetString(stream.ToArray());
            _ht.Response.Write(output);
            SetCompleate();
        }

        private void StartAsyncTask(Object workItemState) {
            try {
                object[] objects = workItemState as object[];
                Type compiledType = WebServiceParser.GetCompiledType((string)objects[0], _ht);
                Type realType = objects[1].GetType();
                protocol.SetContext(compiledType, _ht, _ht.Request, _ht.Response);
                System.Reflection.MethodInfo methodInfo = protocol.GetMyMethod(realType);
                if (methodInfo != null) methodInfo.Invoke(objects[1], protocol.ReadParameters());
            }
            catch (Exception ex) {
                MemoryStream stream = new MemoryStream();
                protocol.WriteException(ex, stream);
                string output = Encoding.UTF8.GetString(stream.ToArray());
                if (string.IsNullOrEmpty(output)) throw;
                _ht.Response.Write(output);
                SetCompleate();
            }
        }
        public void SetCompleate() {
            _completed = true;
            _callback(this);
        }
    }
}
