﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Permissions;
using System.Text;
using System.Threading;
using System.Web;
using System.Web.Caching;
using System.Web.Services;
using System.Web.Services.Protocols;

namespace MySoapLib {
    [PermissionSet(SecurityAction.LinkDemand, Name = "FullTrust"), PermissionSet(SecurityAction.InheritanceDemand, Name = "FullTrust")]
    public abstract class ServerProtocol {
        // Fields
        private HttpContext context;
        private WebMethodAttribute methodAttr;
        private HttpRequest request;
        private HttpResponse response;
        private static object s_InternalSyncObject;
        private object target;
        private Type type;

        // Methods
        protected ServerProtocol() {
        }

        protected void AddToCache(Type protocolType, Type serverType, object value) {
            HttpRuntime.Cache.Insert(this.CreateKey(protocolType, serverType), value, null, Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration, CacheItemPriority.NotRemovable, null);
        }

        private string CreateKey(Type protocolType, Type serverType) {
            string fullName = protocolType.FullName;
            string str2 = serverType.FullName;
            string str3 = serverType.TypeHandle.Value.ToString();
            string urlKeyComponent = this.GetUrlKeyComponent();
            int capacity = (((fullName.Length + urlKeyComponent.Length) + str2.Length) + str3.Length) + 2;
            StringBuilder builder = new StringBuilder(capacity);
            builder.Append(fullName);
            builder.Append('[');
            builder.Append(urlKeyComponent);
            builder.Append(']');
            builder.Append(str2);
            builder.Append(str3);
            return builder.ToString();
        }

        internal virtual void CreateServerInstance() {
            //this.target = Activator.CreateInstance(this.ServerType.Type);
            WebService target = this.target as WebService;
            if (target != null) {
             //   target.SetContext(this.context);
            }
        }

        public System.Reflection.MethodInfo GetMyMethod(Type type)
        {
            return type.GetMethod(this.MethodInfo.MethodInfo.Name);
        }

        internal virtual void DisposeServerInstance() {
            if (this.target != null) {
                IDisposable target = this.target as IDisposable;
                if (target != null) {
                    target.Dispose();
                }
                this.target = null;
            }
        }

        internal string GenerateFaultString(Exception e) {
            return this.GenerateFaultString(e, false);
        }

        internal string GenerateFaultString(Exception e, bool htmlEscapeMessage) {
            bool flag = (this.Context != null) && !this.Context.IsCustomErrorEnabled;
            if (flag && !htmlEscapeMessage) {
                return e.ToString();
            }
            StringBuilder builder = new StringBuilder();
            if (flag) {
                GenerateFaultString(e, builder);
            }
            else {
                for (Exception exception = e; exception != null; exception = exception.InnerException) {
                    string name = htmlEscapeMessage ? HttpUtility.HtmlEncode(exception.Message) : exception.Message;
                    if (name.Length == 0) {
                        name = e.GetType().Name;
                    }
                    builder.Append(name);
                    if (exception.InnerException != null) {
                        builder.Append(" ---> ");
                    }
                }
            }
            return builder.ToString();
        }

        private static void GenerateFaultString(Exception e, StringBuilder builder) {
            builder.Append(e.GetType().FullName);
            if ((e.Message != null) && (e.Message.Length > 0)) {
                builder.Append(": ");
                builder.Append(HttpUtility.HtmlEncode(e.Message));
            }
            if (e.InnerException != null) {
                builder.Append(" ---> ");
                GenerateFaultString(e.InnerException, builder);
                builder.Append(Environment.NewLine);
                builder.Append("   ");
                builder.Append(Res.GetString("StackTraceEnd"));
            }
            if (e.StackTrace != null) {
                builder.Append(Environment.NewLine);
                builder.Append(e.StackTrace);
            }
        }

        protected object GetFromCache(Type protocolType, Type serverType) {
            return HttpRuntime.Cache.Get(this.CreateKey(protocolType, serverType));
        }

        internal virtual string GetUrlKeyComponent() {
            return this.Request.Url.GetLeftPart(UriPartial.Path);
        }

        internal abstract bool Initialize();
        public abstract object[] ReadParameters();
        protected Stream _stream;
        public void SetContext(Type type, HttpContext context, HttpRequest request, HttpResponse response) {
            SetContext(type,context,request,response,null);
        }
        public void SetContext(Type type, HttpContext context, HttpRequest request, HttpResponse response, Stream stream) {
            this.type = type;
            this.context = context;
            this.request = request;
            this.response = response;
            _stream = stream;
            this.Initialize();
        }

        internal static void SetHttpResponseStatusCode(HttpResponse httpResponse, int statusCode) {
            httpResponse.TrySkipIisCustomErrors = true;
            httpResponse.StatusCode = statusCode;
        }

        public virtual bool WriteException(Exception e, Stream outputStream) {
            return false;
        }

        internal void WriteOneWayResponse() {
            this.context.Response.ContentType = null;
            this.Response.StatusCode = 0xca;
        }

        public abstract void WriteReturns(object[] returns, Stream outputStream);

        // Properties
        protected internal HttpContext Context {
            get {
                return this.context;
            }
        }

        internal static object InternalSyncObject {
            get {
                if (s_InternalSyncObject == null) {
                    object obj2 = new object();
                    Interlocked.CompareExchange(ref s_InternalSyncObject, obj2, null);
                }
                return s_InternalSyncObject;
            }
        }

        internal abstract bool IsOneWay { get; }

        internal WebMethodAttribute MethodAttribute {
            get {
                if (this.methodAttr == null) {
                    this.methodAttr = this.MethodInfo.MethodAttribute;
                }
                return this.methodAttr;
            }
        }

        internal abstract LogicalMethodInfo MethodInfo { get; }

        internal virtual Exception OnewayInitException {
            get {
                return null;
            }
        }

        protected internal HttpRequest Request {
            get {
                return this.request;
            }
        }

        protected internal HttpResponse Response {
            get {
                return this.response;
            }
        }

        internal abstract ServerType ServerType { get; }

        protected internal virtual object Target {
            get {
                return this.target;
            }
        }

        internal Type Type {
            get {
                return this.type;
            }
        }
    }

 

}
