﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Web;
using System.Web.Services.Protocols;

namespace MySoapLib {
    public sealed class SoapServerMessage : SoapMessage {
        // Fields
        internal SoapExtension[] allExtensions;
        internal SoapExtension[] highPriConfigExtensions;
        internal SoapExtension[] otherExtensions;
        private SoapServerProtocol protocol;

        // Methods
        internal SoapServerMessage(SoapServerProtocol protocol) {
            this.protocol = protocol;
        }

        protected override void EnsureInStage() {
            base.EnsureStage(SoapMessageStage.AfterDeserialize);
        }

        protected override void EnsureOutStage() {
            base.EnsureStage(SoapMessageStage.BeforeSerialize);
        }

        // Properties
        public override string Action {
            get {
                return this.protocol.ServerMethod.action;
            }
        }

        public override LogicalMethodInfo MethodInfo {
            get {
                return this.protocol.MethodInfo;
            }
        }

        public override bool OneWay {
            get {
                return this.protocol.ServerMethod.oneWay;
            }
        }

        public object Server {
            get {
                base.EnsureStage(SoapMessageStage.AfterDeserialize | SoapMessageStage.BeforeSerialize);
                return this.protocol.Target;
            }
        }

        [ComVisible(false)]
        public override SoapProtocolVersion SoapVersion {
            get {
                return this.protocol.Version;
            }
        }

        public override string Url {
            get {
                return this.protocol.Request.Url.GetComponents(UriComponents.AbsoluteUri, UriFormat.UriEscaped).Replace("#", "%23");
            }
        }
    }


}
