﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Security.Permissions;
using System.Text;
using System.Threading;
using System.Web;
using System.Web.Caching;
using System.Web.Services;
using System.Web.Services.Configuration;
using System.Web.Services.Description;
using System.Web.Services.Protocols;
using System.Xml;
using System.Xml.Serialization;

namespace MySoapLib {
    [PermissionSet(SecurityAction.InheritanceDemand, Name = "FullTrust"), PermissionSet(SecurityAction.LinkDemand, Name = "FullTrust")]
    public class SoapServerProtocol : ServerProtocol {
        // Fields
        private SoapServerProtocolHelper helper;
        private bool isOneWay;
        private SoapServerMessage message;
        private Exception onewayInitException;
        private WebServiceProtocols protocolsSupported = WebServicesSection.Current.EnabledProtocols;
        private SoapServerMethod serverMethod;
        private SoapServerType serverType;
        private SoapProtocolVersion version;
        public string SoapAction = "http://www.fetion.com.cn/internetfetionservice/Reg";
        // Methods
        public SoapServerProtocol() {
        }

        private void CheckHelperVersion() {
            if (this.helper.RequestNamespace != null) {
                if (this.helper.RequestNamespace != this.helper.EnvelopeNs) {
                    string requestNamespace = this.helper.RequestNamespace;
                    if (this.IsSupported(WebServiceProtocols.HttpSoap)) {
                        this.SetHelper(new Soap11ServerProtocolHelper(this));
                    }
                    else {
                        this.SetHelper(new Soap12ServerProtocolHelper(this));
                    }
                    throw new SoapException(Res.GetString("WebInvalidEnvelopeNamespace", new object[] { requestNamespace, this.helper.EnvelopeNs }), SoapException.VersionMismatchFaultCode);
                }
                if (!this.IsSupported(this.helper.Protocol)) {
                    string str2 = this.helper.RequestNamespace;
                    string str3 = this.IsSupported(WebServiceProtocols.HttpSoap) ? "http://schemas.xmlsoap.org/soap/envelope/" : "http://www.w3.org/2003/05/soap-envelope";
                    this.SetHelper(new Soap11ServerProtocolHelper(this));
                    throw new SoapException(Res.GetString("WebInvalidEnvelopeNamespace", new object[] { str2, str3 }), SoapException.VersionMismatchFaultCode);
                }
            }
        }

        private static Array CombineExtensionsHelper(Array array1, Array array2, Type elementType) {
            if (array1 == null) {
                return array2;
            }
            if (array2 == null) {
                return array1;
            }
            int num = array1.Length + array2.Length;
            if (num == 0) {
                return null;
            }
            Array destinationArray = null;
            if (elementType == typeof(SoapReflectedExtension)) {
                destinationArray = new SoapReflectedExtension[num];
            }
            else if (elementType == typeof(SoapExtension)) {
                destinationArray = new SoapExtension[num];
            }
            else {
                if (elementType != typeof(object)) {
                    throw new ArgumentException(Res.GetString("ElementTypeMustBeObjectOrSoapExtensionOrSoapReflectedException"), "elementType");
                }
                destinationArray = new object[num];
            }
            Array.Copy(array1, 0, destinationArray, 0, array1.Length);
            Array.Copy(array2, 0, destinationArray, array1.Length, array2.Length);
            return destinationArray;
        }

        internal override void CreateServerInstance() {
            base.CreateServerInstance();
            this.message.SetStage(SoapMessageStage.AfterDeserialize);
            this.message.RunExtensions(this.message.allExtensions, true);
            SoapHeaderHandling.SetHeaderMembers(this.message.Headers, this.Target, this.serverMethod.inHeaderMappings, SoapHeaderDirection.In, false);
        }

        private static XmlElement CreateUpgradeEnvelope(XmlDocument doc, string prefix, string envelopeNs) {
            XmlElement element = doc.CreateElement("soap12", "SupportedEnvelope", "http://www.w3.org/2003/05/soap-envelope");
            XmlAttribute node = doc.CreateAttribute("xmlns", prefix, "http://www.w3.org/2000/xmlns/");
            node.Value = envelopeNs;
            XmlAttribute attribute2 = doc.CreateAttribute("qname");
            attribute2.Value = prefix + ":Envelope";
            element.Attributes.Append(attribute2);
            element.Attributes.Append(node);
            return element;
        }

        internal SoapUnknownHeader CreateUpgradeHeader() {
            XmlDocument doc = new XmlDocument();
            XmlElement element = doc.CreateElement("soap12", "Upgrade", "http://www.w3.org/2003/05/soap-envelope");
            if (this.IsSupported(WebServiceProtocols.HttpSoap)) {
                element.AppendChild(CreateUpgradeEnvelope(doc, "soap", "http://schemas.xmlsoap.org/soap/envelope/"));
            }
            if (this.IsSupported(WebServiceProtocols.HttpSoap12)) {
                element.AppendChild(CreateUpgradeEnvelope(doc, "soap12", "http://www.w3.org/2003/05/soap-envelope"));
            }
            SoapUnknownHeader header = new SoapUnknownHeader();
            header.Element = element;
            return header;
        }

        [PermissionSet(SecurityAction.InheritanceDemand, Name = "FullTrust")]
        protected virtual XmlReader GetReaderForMessage(SoapServerMessage message, int bufferSize) {
            XmlTextReader reader;
            Encoding encoding = RequestResponseUtils.GetEncoding2(message.ContentType);
            if (bufferSize < 0x200) {
                bufferSize = 0x200;
            }
            int readTimeout = WebServicesSection.Current.SoapEnvelopeProcessing.ReadTimeout;
            long num2 = (readTimeout < 0) ? 0L : (readTimeout * 0x989680L);
            long ticks = DateTime.UtcNow.Ticks;
            long timeout = ((0x7fffffffffffffffL - num2) <= ticks) ? 0x7fffffffffffffffL : (ticks + num2);
            if (encoding != null) {
                if (timeout == 0x7fffffffffffffffL) {
                    reader = new XmlTextReader(new StreamReader(message.Stream, encoding, true, bufferSize));
                }
                else {
                    reader = new SoapEnvelopeReader(new StreamReader(message.Stream, encoding, true, bufferSize), timeout);
                }
            }
            else if (timeout == 0x7fffffffffffffffL) {
                reader = new XmlTextReader(message.Stream);
            }
            else {
                reader = new SoapEnvelopeReader(message.Stream, timeout);
            }
            reader.ProhibitDtd = true;
            reader.Normalization = true;
            reader.XmlResolver = null;
            return reader;
        }

        internal override string GetUrlKeyComponent() {
            return string.Empty;
        }

        [PermissionSet(SecurityAction.InheritanceDemand, Name = "FullTrust")]
        protected virtual XmlWriter GetWriterForMessage(SoapServerMessage message, int bufferSize) {
            if (bufferSize < 0x200) {
                bufferSize = 0x200;
            }
            return new XmlTextWriter(new StreamWriter(message.Stream, new UTF8Encoding(false), bufferSize));
        }

        internal XmlReader GetXmlReader() {
            Encoding encoding = RequestResponseUtils.GetEncoding2(this.Message.ContentType);
            if (((((this.serverMethod != null) && ((this.serverMethod.wsiClaims & WsiProfiles.BasicProfile1_1) != WsiProfiles.None)) && (this.Version != SoapProtocolVersion.Soap12)) && (encoding != null)) && (!(encoding is UTF8Encoding) && !(encoding is UnicodeEncoding))) {
                throw new InvalidOperationException(Res.GetString("WebWsiContentTypeEncoding"));
            }
            int contentLength = (int) (_stream == null? Request.ContentLength:_stream.Length);

            XmlReader readerForMessage = this.GetReaderForMessage(this.Message, RequestResponseUtils.GetBufferSize(contentLength));

            if (readerForMessage == null) {
                throw new InvalidOperationException(Res.GetString("WebNullReaderForMessage"));
            }
            return readerForMessage;
        }

        private void GuessVersion() {
            if (this.IsSupported(WebServiceProtocols.AnyHttpSoap)) {
                if ((base.Request.Headers["SOAPAction"] == null) || ContentType.MatchesBase(base.Request.ContentType, "application/soap+xml")) {
                    this.SetHelper(new Soap12ServerProtocolHelper(this));
                }
                else {
                    this.SetHelper(new Soap11ServerProtocolHelper(this));
                }
            }
            else if (this.IsSupported(WebServiceProtocols.HttpSoap)) {
                this.SetHelper(new Soap11ServerProtocolHelper(this));
            }
            else if (this.IsSupported(WebServiceProtocols.HttpSoap12)) {
                this.SetHelper(new Soap12ServerProtocolHelper(this));
            }
        }

        internal override bool Initialize() {
            this.GuessVersion();
            this.message = new SoapServerMessage(this);
            this.onewayInitException = null;
            this.serverType = (SoapServerType)base.GetFromCache(typeof(SoapServerProtocol), base.Type);
            if (this.serverType == null) {
                lock (ServerProtocol.InternalSyncObject) {
                    this.serverType = (SoapServerType)base.GetFromCache(typeof(SoapServerProtocol), base.Type);
                    if (this.serverType == null) {
                        this.serverType = new SoapServerType(base.Type, this.protocolsSupported);
                        base.AddToCache(typeof(SoapServerProtocol), base.Type, this.serverType);
                    }
                }
            }
            Exception innerException = null;
            try {
                this.message.highPriConfigExtensions = SoapMessage.InitializeExtensions(this.serverType.HighPriExtensions, this.serverType.HighPriExtensionInitializers);
                this.message.highPriConfigExtensions = this.ModifyInitializedExtensions(PriorityGroup.High, this.message.highPriConfigExtensions);
                if(_stream != null)
                    this.message.SetStream(_stream);
                else
                    this.message.SetStream(Request.InputStream);
                this.message.InitExtensionStreamChain(this.message.highPriConfigExtensions);
                this.message.SetStage(SoapMessageStage.BeforeDeserialize);
                this.message.ContentType = base.Request.ContentType;
                this.message.ContentEncoding = base.Request.Headers["Content-Encoding"];
                this.message.RunExtensions(this.message.highPriConfigExtensions, false);
                innerException = this.message.Exception;
            }
            catch (Exception exception2) {
                if (((exception2 is ThreadAbortException) || (exception2 is StackOverflowException)) || (exception2 is OutOfMemoryException)) {
                    throw;
                }
               
                innerException = exception2;
            }
            catch {
                innerException = new Exception(Res.GetString("NonClsCompliantException"));
            }
            this.message.allExtensions = this.message.highPriConfigExtensions;
            this.GuessVersion();
            try {
                this.serverMethod = this.RouteRequest(this.message);
                if (this.serverMethod == null) {
                    throw new SoapException(Res.GetString("UnableToHandleRequest0"), new XmlQualifiedName("Server", "http://schemas.xmlsoap.org/soap/envelope/"));
                }
            }
            catch (Exception exception3) {
                if (((exception3 is ThreadAbortException) || (exception3 is StackOverflowException)) || (exception3 is OutOfMemoryException)) {
                    throw;
                }
                if (this.helper.RequestNamespace != null) {
                    this.SetHelper(SoapServerProtocolHelper.GetHelper(this, this.helper.RequestNamespace));
                }
                this.CheckHelperVersion();
                throw;
            }
            catch {
                if (this.helper.RequestNamespace != null) {
                    this.SetHelper(SoapServerProtocolHelper.GetHelper(this, this.helper.RequestNamespace));
                }
                this.CheckHelperVersion();
                throw;
            }
            this.isOneWay = this.serverMethod.oneWay;
            if (innerException == null) {
                try {
                    SoapReflectedExtension[] reflectedExtensions = (SoapReflectedExtension[])CombineExtensionsHelper(this.serverMethod.extensions, this.serverType.LowPriExtensions, typeof(SoapReflectedExtension));
                    object[] extensionInitializers = (object[])CombineExtensionsHelper(this.serverMethod.extensionInitializers, this.serverType.LowPriExtensionInitializers, typeof(object));
                    this.message.otherExtensions = SoapMessage.InitializeExtensions(reflectedExtensions, extensionInitializers);
                    this.message.otherExtensions = this.ModifyInitializedExtensions(PriorityGroup.Low, this.message.otherExtensions);
                    this.message.allExtensions = (SoapExtension[])CombineExtensionsHelper(this.message.highPriConfigExtensions, this.message.otherExtensions, typeof(SoapExtension));
                }
                catch (Exception exception4) {
                    if (((exception4 is ThreadAbortException) || (exception4 is StackOverflowException)) || (exception4 is OutOfMemoryException)) {
                        throw;
                    }
                  
                    innerException = exception4;
                }
                catch {
                    innerException = new Exception(Res.GetString("NonClsCompliantException"));
                }
            }
            if (innerException != null) {
                if (!this.isOneWay) {
                    if (innerException is SoapException) {
                        throw innerException;
                    }
                    throw SoapException.Create(this.Version, Res.GetString("WebConfigExtensionError"), new XmlQualifiedName("Server", "http://schemas.xmlsoap.org/soap/envelope/"), innerException);
                }
                this.onewayInitException = innerException;
            }
            return true;
        }

        internal bool IsSupported(WebServiceProtocols protocol) {
            return ((this.protocolsSupported & protocol) == protocol);
        }

        protected virtual SoapExtension[] ModifyInitializedExtensions(PriorityGroup group, SoapExtension[] extensions) {
            return extensions;
        }

        public override object[] ReadParameters() {
            object[] objArray2;
            this.message.InitExtensionStreamChain(this.message.otherExtensions);
            this.message.RunExtensions(this.message.otherExtensions, true);
            if (!ContentType.IsSoap(this.message.ContentType)) {
                throw new SoapException(Res.GetString("WebRequestContent", new object[] { this.message.ContentType, this.helper.HttpContentType }), new XmlQualifiedName("Client", "http://schemas.xmlsoap.org/soap/envelope/"), new SoapFaultSubCode(Soap12FaultCodes.UnsupportedMediaTypeFaultCode));
            }
            XmlReader xmlReader = null;
            try {
                xmlReader = this.GetXmlReader();
                xmlReader.MoveToContent();
                this.SetHelper(SoapServerProtocolHelper.GetHelper(this, xmlReader.NamespaceURI));
            }
            catch (XmlException exception) {
                throw new SoapException(Res.GetString("WebRequestUnableToRead"), new XmlQualifiedName("Client", "http://schemas.xmlsoap.org/soap/envelope/"), exception);
            }
            this.CheckHelperVersion();
            if ((this.version == SoapProtocolVersion.Soap11) && !ContentType.MatchesBase(this.message.ContentType, this.helper.HttpContentType)) {
                throw new SoapException(Res.GetString("WebRequestContent", new object[] { this.message.ContentType, this.helper.HttpContentType }), new XmlQualifiedName("Client", "http://schemas.xmlsoap.org/soap/envelope/"), new SoapFaultSubCode(Soap12FaultCodes.UnsupportedMediaTypeFaultCode));
            }
            if (this.message.Exception != null) {
                throw this.message.Exception;
            }
            try {
                object[] objArray;
                if (!xmlReader.IsStartElement("Envelope", this.helper.EnvelopeNs)) {
                    throw new InvalidOperationException(Res.GetString("WebMissingEnvelopeElement"));
                }
                if (xmlReader.IsEmptyElement) {
                    throw new InvalidOperationException(Res.GetString("WebMissingBodyElement"));
                }
                int depth = xmlReader.Depth;
                xmlReader.ReadStartElement("Envelope", this.helper.EnvelopeNs);
                xmlReader.MoveToContent();
                bool checkRequiredHeaders = ((this.serverMethod.wsiClaims & WsiProfiles.BasicProfile1_1) != WsiProfiles.None) && (this.version != SoapProtocolVersion.Soap12);
                string str = new SoapHeaderHandling().ReadHeaders(xmlReader, this.serverMethod.inHeaderSerializer, this.message.Headers, this.serverMethod.inHeaderMappings, SoapHeaderDirection.In, this.helper.EnvelopeNs, (this.serverMethod.use == SoapBindingUse.Encoded) ? this.helper.EncodingNs : null, checkRequiredHeaders);
                if (str != null) {
                    throw new SoapHeaderException(Res.GetString("WebMissingHeader", new object[] { str }), new XmlQualifiedName("MustUnderstand", "http://schemas.xmlsoap.org/soap/envelope/"));
                }
                if (!xmlReader.IsStartElement("Body", this.helper.EnvelopeNs)) {
                    throw new InvalidOperationException(Res.GetString("WebMissingBodyElement"));
                }
                xmlReader.ReadStartElement("Body", this.helper.EnvelopeNs);
                xmlReader.MoveToContent();
                bool flag2 = this.serverMethod.use == SoapBindingUse.Encoded;
             
               
              
                    objArray = (object[])this.serverMethod.parameterSerializer.Deserialize(xmlReader, flag2 ? this.helper.EncodingNs : null);
       
              
                while ((depth < xmlReader.Depth) && xmlReader.Read()) {
                }
                if (xmlReader.NodeType == XmlNodeType.EndElement) {
                    xmlReader.Read();
                }
                this.message.SetParameterValues(objArray);
                objArray2 = objArray;
            }
            catch (SoapException) {
                throw;
            }
            catch (Exception exception2) {
                if (((exception2 is ThreadAbortException) || (exception2 is StackOverflowException)) || (exception2 is OutOfMemoryException)) {
                    throw;
                }
                throw new SoapException(Res.GetString("WebRequestUnableToRead"), new XmlQualifiedName("Client", "http://schemas.xmlsoap.org/soap/envelope/"), exception2);
            }
            catch {
                throw new SoapException(Res.GetString("WebRequestUnableToRead"), new XmlQualifiedName("Client", "http://schemas.xmlsoap.org/soap/envelope/"), new Exception(Res.GetString("NonClsCompliantException")));
            }
            return objArray2;
        }

        protected virtual SoapServerMethod RouteRequest(SoapServerMessage message) {
            return this.helper.RouteRequest();
        }

        private void SetHelper(SoapServerProtocolHelper helper) {
            this.helper = helper;
            this.version = helper.Version;
            base.Context.Items["WebServiceSoapVersion"] = helper.Version;
        }

        public override bool WriteException(Exception e, Stream outputStream) {
            SoapException exception;
            if (this.message == null) {
                return false;
            }
            this.message.Headers.Clear();
            if ((this.serverMethod != null) && (this.Target != null)) {
                SoapHeaderHandling.GetHeaderMembers(this.message.Headers, this.Target, this.serverMethod.outHeaderMappings, SoapHeaderDirection.Fault, false);
            }
            if (e is SoapException) {
                exception = (SoapException)e;
            }
            else if (((this.serverMethod != null) && this.serverMethod.rpc) && ((this.helper.Version == SoapProtocolVersion.Soap12) && (e is ArgumentException))) {
                exception = SoapException.Create(this.Version, Res.GetString("WebRequestUnableToProcess"), new XmlQualifiedName("Client", "http://schemas.xmlsoap.org/soap/envelope/"), null, null, null, new SoapFaultSubCode(Soap12FaultCodes.RpcBadArgumentsFaultCode), e);
            }
            else {
                exception = SoapException.Create(this.Version, Res.GetString("WebRequestUnableToProcess"), new XmlQualifiedName("Server", "http://schemas.xmlsoap.org/soap/envelope/"), e);
            }
            if (SoapException.IsVersionMismatchFaultCode(exception.Code) && this.IsSupported(WebServiceProtocols.HttpSoap12)) {
                SoapUnknownHeader header = this.CreateUpgradeHeader();
                if (header != null) {
                    this.Message.Headers.Add(header);
                }
            }
            base.Response.ClearHeaders();
            base.Response.Clear();
            HttpStatusCode statusCode = this.helper.SetResponseErrorCode(base.Response, exception);
            bool flag = false;
            SoapExtensionStream extensionStream = new SoapExtensionStream();
            if (this.message.allExtensions != null) {
                this.message.SetExtensionStream(extensionStream);
            }
            try {
                this.message.InitExtensionStreamChain(this.message.allExtensions);
            }
            catch (Exception exception2) {
                if (((exception2 is ThreadAbortException) || (exception2 is StackOverflowException)) || (exception2 is OutOfMemoryException)) {
                    throw;
                }
              
                flag = true;
            }
            catch {
                flag = true;
            }
            this.message.SetStage(SoapMessageStage.BeforeSerialize);
            this.message.ContentType = ContentType.Compose(this.helper.HttpContentType, Encoding.UTF8);
            this.message.Exception = exception;
            if (!flag) {
                try {
                    this.message.RunExtensions(this.message.allExtensions, false);
                }
                catch (Exception exception3) {
                    if (((exception3 is ThreadAbortException) || (exception3 is StackOverflowException)) || (exception3 is OutOfMemoryException)) {
                        throw;
                    }
                  
                    flag = true;
                }
                catch {
                    flag = true;
                }
            }
            this.message.SetStream(outputStream);
            base.Response.ContentType = this.message.ContentType;
            if ((this.message.ContentEncoding != null) && (this.message.ContentEncoding.Length > 0)) {
                base.Response.AppendHeader("Content-Encoding", this.message.ContentEncoding);
            }
            XmlWriter writerForMessage = this.GetWriterForMessage(this.message, 0x200);
            if (writerForMessage == null) {
                throw new InvalidOperationException(Res.GetString("WebNullWriterForMessage"));
            }
            this.helper.WriteFault(writerForMessage, this.message.Exception, statusCode);
            if (!flag) {
                SoapException exception4 = null;
                try {
                    this.message.SetStage(SoapMessageStage.AfterSerialize);
                    this.message.RunExtensions(this.message.allExtensions, false);
                }
                catch (Exception exception5) {
                    if (((exception5 is ThreadAbortException) || (exception5 is StackOverflowException)) || (exception5 is OutOfMemoryException)) {
                        throw;
                    }
                   
                    if (!extensionStream.HasWritten) {
                        exception4 = SoapException.Create(this.Version, Res.GetString("WebExtensionError"), new XmlQualifiedName("Server", "http://schemas.xmlsoap.org/soap/envelope/"), exception5);
                    }
                }
                catch {
                    if (!extensionStream.HasWritten) {
                        exception4 = SoapException.Create(this.Version, Res.GetString("WebExtensionError"), new XmlQualifiedName("Server", "http://schemas.xmlsoap.org/soap/envelope/"), null);
                    }
                    return false;
                }
                if (exception4 != null) {
                    base.Response.ContentType = ContentType.Compose("text/plain", Encoding.UTF8);
                    StreamWriter writer2 = new StreamWriter(outputStream, new UTF8Encoding(false));
                    writer2.WriteLine(base.GenerateFaultString(this.message.Exception));
                    writer2.Flush();
                }
            }
            return true;
        }

        private bool WriteException_TryWriteFault(SoapServerMessage message, Stream outputStream, HttpStatusCode statusCode, bool disableExtensions) {
            return true;
        }

        public override void WriteReturns(object[] returnValues, Stream outputStream) {
            if (!this.serverMethod.oneWay) {
                bool isEncoded = this.serverMethod.use == SoapBindingUse.Encoded;
                SoapHeaderHandling.EnsureHeadersUnderstood(this.message.Headers);
                this.message.Headers.Clear();
                SoapHeaderHandling.GetHeaderMembers(this.message.Headers, this.Target, this.serverMethod.outHeaderMappings, SoapHeaderDirection.Out, false);
                if (this.message.allExtensions != null) {
                    this.message.SetExtensionStream(new SoapExtensionStream());
                }
                this.message.InitExtensionStreamChain(this.message.allExtensions);
                this.message.SetStage(SoapMessageStage.BeforeSerialize);
                this.message.ContentType = ContentType.Compose(this.helper.HttpContentType, Encoding.UTF8);
                this.message.SetParameterValues(returnValues);
                this.message.RunExtensions(this.message.allExtensions, true);
                this.message.SetStream(outputStream);
                base.Response.ContentType = this.message.ContentType;
                if ((this.message.ContentEncoding != null) && (this.message.ContentEncoding.Length > 0)) {
                    base.Response.AppendHeader("Content-Encoding", this.message.ContentEncoding);
                }
                XmlWriter writerForMessage = this.GetWriterForMessage(this.message, 0x400);
                if (writerForMessage == null) {
                    throw new InvalidOperationException(Res.GetString("WebNullWriterForMessage"));
                }
                writerForMessage.WriteStartDocument();
                writerForMessage.WriteStartElement("soap", "Envelope", this.helper.EnvelopeNs);
                writerForMessage.WriteAttributeString("xmlns", "soap", null, this.helper.EnvelopeNs);
                if (isEncoded) {
                    writerForMessage.WriteAttributeString("xmlns", "soapenc", null, this.helper.EncodingNs);
                    writerForMessage.WriteAttributeString("xmlns", "tns", null, this.serverType.serviceNamespace);
                    writerForMessage.WriteAttributeString("xmlns", "types", null, SoapReflector.GetEncodedNamespace(this.serverType.serviceNamespace, this.serverType.serviceDefaultIsEncoded));
                }
                if (this.serverMethod.rpc && (this.version == SoapProtocolVersion.Soap12)) {
                    writerForMessage.WriteAttributeString("xmlns", "rpc", null, "http://www.w3.org/2003/05/soap-rpc");
                }
                writerForMessage.WriteAttributeString("xmlns", "xsi", null, "http://www.w3.org/2001/XMLSchema-instance");
                writerForMessage.WriteAttributeString("xmlns", "xsd", null, "http://www.w3.org/2001/XMLSchema");
                SoapHeaderHandling.WriteHeaders(writerForMessage, this.serverMethod.outHeaderSerializer, this.message.Headers, this.serverMethod.outHeaderMappings, SoapHeaderDirection.Out, isEncoded, this.serverType.serviceNamespace, this.serverType.serviceDefaultIsEncoded, this.helper.EnvelopeNs);
                writerForMessage.WriteStartElement("Body", this.helper.EnvelopeNs);
                if (isEncoded && (this.version != SoapProtocolVersion.Soap12)) {
                    writerForMessage.WriteAttributeString("soap", "encodingStyle", null, this.helper.EncodingNs);
                }
                
                this.serverMethod.returnSerializer.Serialize(writerForMessage, returnValues, null, isEncoded ? this.helper.EncodingNs : null);
               
                writerForMessage.WriteEndElement();
                writerForMessage.WriteEndElement();
                writerForMessage.Flush();
                this.message.SetStage(SoapMessageStage.AfterSerialize);
                
                //wawa
                //this.message.RunExtensions(this.message.allExtensions, true);
            }
        }

        // Properties
        internal override bool IsOneWay {
            get {
                return this.isOneWay;
            }
        }

        internal SoapServerMessage Message {
            get {
                return this.message;
            }
        }

        internal override LogicalMethodInfo MethodInfo {
            get {
                return this.serverMethod.methodInfo;
            }
        }

        internal override Exception OnewayInitException {
            get {
                return this.onewayInitException;
            }
        }

        internal SoapServerMethod ServerMethod {
            get {
                return this.serverMethod;
            }
        }

        internal override ServerType ServerType {
            get {
                return this.serverType;
            }
        }

        internal SoapProtocolVersion Version {
            get {
                return this.version;
            }
        }

        // Nested Types
        internal class SoapEnvelopeReader : XmlTextReader {
            // Fields
            private long readerTimedout;

            // Methods
            internal SoapEnvelopeReader(Stream input, long timeout)
                : base(input) {
                this.readerTimedout = timeout;
            }

            internal SoapEnvelopeReader(TextReader input, long timeout)
                : base(input) {
                this.readerTimedout = timeout;
            }

            private void CheckTimeout() {
                if (DateTime.UtcNow.Ticks > this.readerTimedout) {
                    throw new InvalidOperationException(Res.GetString("WebTimeout"));
                }
            }

            public override XmlNodeType MoveToContent() {
                this.CheckTimeout();
                string temp = this.Name;
                return base.MoveToContent();
            }

            public override bool MoveToNextAttribute() {
                this.CheckTimeout();
                return base.MoveToNextAttribute();
            }

            public override bool Read() {
                this.CheckTimeout();
                string temp = this.Name;
                return base.Read();
            }
        }
    }




}
