﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Net;
using System.Net.Mime;
using System.Security.Permissions;
using System.Text;
using System.Web;
using System.Web.Services.Description;
using System.Web.Services.Protocols;
using System.Xml;

namespace MySoapLib {
    public class SoapTest {
        public SoapTest(HttpWebClientProtocol clientProtocol)
        {
            Type type = clientProtocol.GetType();
            this.clientType = new SoapClientType(type);
        }

        public string Invoke(string methodName, object[] parameters) {
            SoapClientMessage message = this.BeforeSerialize(methodName, parameters);
            
            Stream requestStream = new MemoryStream();
            message.SetStream(requestStream);
            Serialize(message);
            requestStream.Seek(0, SeekOrigin.Begin);
            StreamReader reader = new StreamReader(requestStream);
            return reader.ReadToEnd();
        }
        private static Array CombineExtensionsHelper(Array array1, Array array2, Array array3, Type elementType) {
            int num = (array1.Length + array2.Length) + array3.Length;
            if (num == 0) {
                return null;
            }
            Array destinationArray = null;
            if (elementType == typeof(SoapReflectedExtension)) {
                destinationArray = new SoapReflectedExtension[num];
            }
            else {
                if (elementType != typeof(object)) {
                    throw new ArgumentException(Res.GetString("ElementTypeMustBeObjectOrSoapReflectedException"), "elementType");
                }
                destinationArray = new object[num];
            }
            int destinationIndex = 0;
            Array.Copy(array1, 0, destinationArray, destinationIndex, array1.Length);
            destinationIndex += array1.Length;
            Array.Copy(array2, 0, destinationArray, destinationIndex, array2.Length);
            destinationIndex += array2.Length;
            Array.Copy(array3, 0, destinationArray, destinationIndex, array3.Length);
            return destinationArray;
        }



        public string Url = "http://www.abc.com";

        private SoapClientMessage BeforeSerialize(string methodName, object[] parameters) {
            if (parameters == null) {
                throw new ArgumentNullException("parameters");
            }
            SoapClientMethod method = this.clientType.GetMethod(methodName);
            if (method == null) {
                throw new ArgumentException(Res.GetString("WebInvalidMethodName", new object[] { methodName }));
            }
            SoapReflectedExtension[] reflectedExtensions = (SoapReflectedExtension[])CombineExtensionsHelper(this.clientType.HighPriExtensions, method.extensions, this.clientType.LowPriExtensions, typeof(SoapReflectedExtension));
            object[] extensionInitializers = (object[])CombineExtensionsHelper(this.clientType.HighPriExtensionInitializers, method.extensionInitializers, this.clientType.LowPriExtensionInitializers, typeof(object));
            SoapExtension[] extensionArray2 = SoapMessage.InitializeExtensions(reflectedExtensions, extensionInitializers);
            SoapClientMessage message = new SoapClientMessage( method, Url);
            message.initializedExtensions = extensionArray2;
            if (extensionArray2 != null) {
                message.SetExtensionStream(new SoapExtensionStream());
            }
            message.InitExtensionStreamChain(message.initializedExtensions);
            string action = UrlEncoder.EscapeString(method.action, Encoding.UTF8);
            message.SetStage(SoapMessageStage.BeforeSerialize);
            if (this.version == SoapProtocolVersion.Soap12) {
                message.ContentType = ContentType.Compose("application/soap+xml", (RequestEncoding != null) ? RequestEncoding : Encoding.UTF8, action);
            }
            else {
                message.ContentType = ContentType.Compose("text/xml", (RequestEncoding != null) ? RequestEncoding : Encoding.UTF8);
            }
            message.SetParameterValues(parameters);
            SoapHeaderHandling.GetHeaderMembers(message.Headers, this, method.inHeaderMappings, SoapHeaderDirection.In, true);
            message.RunExtensions(message.initializedExtensions, true);
            //request.ContentType = message.ContentType;
            if ((message.ContentEncoding != null) && (message.ContentEncoding.Length > 0)) {
                //request.Headers["Content-Encoding"] = message.ContentEncoding;
            }
            //request.Method = "POST";
            //if ((this.version != SoapProtocolVersion.Soap12) && (request.Headers["SOAPAction"] == null)) {
            //    StringBuilder builder = new StringBuilder(action.Length + 2);
            //    builder.Append('"');
            //    builder.Append(action);
            //    builder.Append('"');
            //    request.Headers.Add("SOAPAction", builder.ToString());
            //}
            return message;
        }


        private Encoding requestEncoding;

        public Encoding RequestEncoding {
            get {
                return this.requestEncoding;
            }
            set {
                this.requestEncoding = value;
            }
        }
 

 

        [PermissionSet(SecurityAction.InheritanceDemand, Name = "FullTrust")]
        protected virtual XmlWriter GetWriterForMessage(SoapClientMessage message, int bufferSize) {
            if (bufferSize < 0x200) {
                bufferSize = 0x200;
            }
            return new XmlTextWriter(new StreamWriter(message.Stream, (RequestEncoding != null) ? RequestEncoding : new UTF8Encoding(false), bufferSize));
        }
        private SoapProtocolVersion version;

        private string EnvelopeNs {
            get {
                if (this.version != SoapProtocolVersion.Soap12) {
                    return "http://schemas.xmlsoap.org/soap/envelope/";
                }
                return "http://www.w3.org/2003/05/soap-envelope";
            }
        }

        private string EncodingNs {
            get {
                if (this.version != SoapProtocolVersion.Soap12) {
                    return "http://schemas.xmlsoap.org/soap/encoding/";
                }
                return "http://www.w3.org/2003/05/soap-encoding";
            }
        }





        private SoapClientType clientType;

        private void Serialize(SoapClientMessage message) {
            Stream stream = message.Stream;
            SoapClientMethod method = message.Method;
            bool isEncoded = method.use == SoapBindingUse.Encoded;
            string envelopeNs = this.EnvelopeNs;
            string encodingNs = this.EncodingNs;
            XmlWriter writerForMessage = this.GetWriterForMessage(message, 0x400);
            if (writerForMessage == null) {
                throw new InvalidOperationException(Res.GetString("WebNullWriterForMessage"));
            }
            writerForMessage.WriteStartDocument();
            writerForMessage.WriteStartElement("soap", "Envelope", envelopeNs);
            writerForMessage.WriteAttributeString("xmlns", "soap", null, envelopeNs);
            if (isEncoded) {
                writerForMessage.WriteAttributeString("xmlns", "soapenc", null, encodingNs);
                writerForMessage.WriteAttributeString("xmlns", "tns", null, this.clientType.serviceNamespace);
                writerForMessage.WriteAttributeString("xmlns", "types", null, SoapReflector.GetEncodedNamespace(this.clientType.serviceNamespace, this.clientType.serviceDefaultIsEncoded)); 
            }
            writerForMessage.WriteAttributeString("xmlns", "xsi", null, "http://www.w3.org/2001/XMLSchema-instance");
            writerForMessage.WriteAttributeString("xmlns", "xsd", null, "http://www.w3.org/2001/XMLSchema");
            SoapHeaderHandling.WriteHeaders(writerForMessage, method.inHeaderSerializer, message.Headers, method.inHeaderMappings, SoapHeaderDirection.In, isEncoded, this.clientType.serviceNamespace, this.clientType.serviceDefaultIsEncoded, envelopeNs);
            writerForMessage.WriteStartElement("Body", envelopeNs);
            if (isEncoded && (this.version != SoapProtocolVersion.Soap12)) {
                writerForMessage.WriteAttributeString("soap", "encodingStyle", null, encodingNs);
            }
            object[] parameterValues = message.GetParameterValues();
         
            method.parameterSerializer.Serialize(writerForMessage, parameterValues, null, isEncoded ? encodingNs : null);
       
            writerForMessage.WriteEndElement();
            writerForMessage.WriteEndElement();
            writerForMessage.Flush();
            message.SetStage(SoapMessageStage.AfterSerialize);
            message.RunExtensions(message.initializedExtensions, true);
        }




    }
}
